@@ -225,7 +225,8 @@ double rk_noncentral_chisquare(rk_state *state, double df, double nonc)
 
 double rk_f(rk_state *state, double dfnum, double dfden)
 {
-    return rk_chisquare(state, dfnum) / rk_chisquare(state, dfden);
+    return ((rk_chisquare(state, dfnum) * dfden) / 
+            (rk_chisquare(state, dfden) * dfnum));
 }
 
 double rk_noncentral_f(rk_state *state, double dfnum, double dfden, double nonc)
@@ -389,7 +390,7 @@ long rk_binomial_inversion(rk_state *state, long n, double p)
         state->q = q = 1.0 - p;
         state->r = qn = exp(n * log(q));
         state->c = np = n*p;
-        state->m = bound = min(n, np + 10.0*sqrt(np));
+        state->m = bound = min(n, np + 10.0*sqrt(np*q + 1));
     } else
     {
         q = state->q;
@@ -447,7 +448,7 @@ long rk_binomial(rk_state *state, long n, double p)
 
 }
 
-long rk_negative_binomial(rk_state *state, long n, double p)
+long rk_negative_binomial(rk_state *state, double n, double p)
 {
     double Y;
 
@@ -524,7 +525,11 @@ long rk_poisson(rk_state *state, double lam)
     {
         return rk_poisson_ptrs(state, lam);
     }
-    else
+    else if (lam == 0) 
+    {
+        return 0;
+    }
+    else 
     {
         return rk_poisson_mult(state, lam);
     }
@@ -545,6 +550,12 @@ double rk_standard_t(rk_state *state, double df)
     return X;
 }
 
+/* Uses the rejection algorithm compared against the wrapped Cauchy
+   distribution suggested by Best and Fisher and documented in 
+   Chapter 9 of Luc's Non-Uniform Random Variate Generation.
+   http://cg.scs.carleton.ca/~luc/rnbookindex.html
+   (but corrected to match the algorithm in R and Python)
+*/
 double rk_vonmises(rk_state *state, double mu, double kappa)
 {
     double r, rho, s;
@@ -563,32 +574,27 @@ double rk_vonmises(rk_state *state, double mu, double kappa)
 
         while (1)
         {
-            U = 2*rk_double(state) - 1;
-            V = 2*rk_double(state) - 1;
+	    U = rk_double(state);
             Z = cos(M_PI*U);
             W = (1 + s*Z)/(s + Z);
             Y = kappa * (s - W);
+            V = rk_double(state);
             if ((Y*(2-Y) - V >= 0) || (log(Y/V)+1 - Y >= 0))
             {
                 break;
             }
         }
 
-        if (U < 0)
+	U = rk_double(state);
+
+	result = acos(W);
+        if (U < 0.5)
         {
-            result = acos(W);
+	    result = -result;
         }
-        else
-        {
-            result = -acos(W);
-        }
-        result += mu + M_PI;
+        result += mu;
         mod = fmod(result, 2*M_PI);
-        if (mod && (mod < 0))
-        {
-            mod += 2*M_PI;
-        }
-        return mod - M_PI;
+        return mod;
     }
 }
 
@@ -671,16 +677,23 @@ long rk_zipf(rk_state *state, double a)
 {
     double T, U, V;
     long X;
-    double b;
-    
-    b = pow(2.0, a-1.0);
+    double am1, b;
+
+    am1 = a - 1.0;
+    b = pow(2.0, am1);
     do
     {
-        U = rk_double(state);
+        U = 1.0-rk_double(state);
         V = rk_double(state);
-        X = (long)floor(pow(U, -1.0/(a-1.0)));
-        T = pow(1.0 + 1.0/X, a-1.0);  
-    } while ((V *X*(T-1.0)/(b-1.0)) > (T/b));
+        X = (long)floor(pow(U, -1.0/am1));
+        /* The real result may be above what can be represented in a signed
+         * long. It will get casted to -sys.maxint-1. Since this is
+         * a straightforward rejection algorithm, we can just reject this value
+         * in the rejection condition below. This function then models a Zipf
+         * distribution truncated to sys.maxint.
+         */
+        T = pow(1.0 + 1.0/X, am1);
+    } while (((V*X*(T-1.0)/(b-1.0)) > (T/b)) || X < 1);
     return X;
 }
 
@@ -737,7 +750,7 @@ long rk_hypergeometric_hyp(rk_state *state, long good, long bad, long sample)
         if (K == 0) break;
     }
     Z = (long)(d2 - Y);
-    if (bad > good) Z = sample - Z;
+    if (good > bad) Z = sample - Z;
     return Z;
 }
 
@@ -790,7 +803,7 @@ long rk_hypergeometric_hrua(rk_state *state, long good, long bad, long sample)
     }
     
     /* this is a correction to HRUA* by Ivan Frohne in rv.py */
-    if (bad > good) Z = m - Z;
+    if (good > bad) Z = m - Z;
     
     /* another fix from rv.py to allow sample to exceed popsize/2 */
     if (m < sample) Z = bad - Z;
@@ -835,14 +848,29 @@ double rk_triangular(rk_state *state, double left, double mode, double right)
 long rk_logseries(rk_state *state, double p)
 {
     double q, r, U, V;
+    long result;
     
     r = log(1.0 - p);
-    
-    V = rk_double(state);
-    if (V >= p) return 1;
-    U = rk_double(state);
-    q = 1.0 - exp(r*U);
-    if (V <= q*q) return (long)floor(1 + log(V)/log(q));
-    if (V <= q) return 1;
-    return 2;
+
+    while (1) {
+        V = rk_double(state);
+        if (V >= p) {
+            return 1;
+        }
+        U = rk_double(state);
+        q = 1.0 - exp(r*U);
+        if (V <= q*q) {
+            result = (long)floor(1 + log(V)/log(q));
+            if (result < 1) {
+                continue;
+            }
+            else {
+                return result;
+            }
+        }
+        if (V <= q) {
+            return 1;
+        }
+        return 2;
+    }
 }
